#include "MemAccess.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/mman.h>


#define MAP_SIZE 4096UL
#define MAP_MASK (MAP_SIZE - 1)


MemAccess::MemAccess()
{
	fd = open("/dev/mem", O_RDWR | O_SYNC);

	if (fd == -1)
		fprintf(stderr, "Error opening /dev/mem.\n");
}

MemAccess::~MemAccess()
{
	if (fd >= 0)
		close(fd);
	fd = 0;
}
	

bool
MemAccess::read32Bits(unsigned long addr, unsigned long& data)
{
    void* map_base = mmap(0,
						  MAP_SIZE,
						  PROT_READ | PROT_WRITE,
						  MAP_SHARED,
						  fd,
						  addr & ~MAP_MASK);
	
    if (map_base == (void *) -1)
	{
		fprintf(stderr, "Error mapping address\n");
		return false;
	}
    
    void* virt_addr = (void*) ((unsigned long) map_base + (addr & MAP_MASK));
	data = *((unsigned long *) virt_addr);

	munmap(map_base, MAP_SIZE);

	return true;
}

bool
MemAccess::write32Bits(unsigned long addr, unsigned long data)
{
    void* map_base = mmap(0,
						  MAP_SIZE,
						  PROT_READ | PROT_WRITE,
						  MAP_SHARED,
						  fd,
						  addr & ~MAP_MASK);
	
    if (map_base == (void *) -1)
	{
		fprintf(stderr, "Error mapping address\n");
		return false;
	}
    
    void* virt_addr = (void*) ((unsigned long) map_base + (addr & MAP_MASK));
	*((unsigned long *) virt_addr) = data;

	munmap(map_base, MAP_SIZE);

	return true;
}

bool
MemAccess::read16Bits(unsigned long addr, unsigned short& data)
{
    void* map_base = mmap(0,
						  MAP_SIZE,
						  PROT_READ | PROT_WRITE,
						  MAP_SHARED,
						  fd,
						  addr & ~MAP_MASK);
	
    if (map_base == (void *) -1)
	{
		fprintf(stderr, "Error mapping address\n");
		return false;
	}
    
    void* virt_addr = (void*) ((unsigned long) map_base + (addr & MAP_MASK));
	data = *((unsigned short *) virt_addr);

	munmap(map_base, MAP_SIZE);

	return true;
}

bool
MemAccess::write16Bits(unsigned long addr, unsigned short data)
{
    void* map_base = mmap(0,
						  MAP_SIZE,
						  PROT_READ | PROT_WRITE,
						  MAP_SHARED,
						  fd,
						  addr & ~MAP_MASK);
	
    if (map_base == (void *) -1)
	{
		fprintf(stderr, "Error mapping address\n");
		return false;
	}
    
    void* virt_addr = (void*) ((unsigned long) map_base + (addr & MAP_MASK));
	*((unsigned short *) virt_addr) = data;

	munmap(map_base, MAP_SIZE);

	return true;
}
